package TwilightMinds.Log;

import java.io.*;
import java.util.*;

/**
 * The FlatFileLog writes to a string-delimited flat file, as might be
 * expected. LogEntry information is recorded:
 * <br><br>
 * <pre>
 *        timestamp     type     message     exception
 * </pre>
 * <br>
 * 'null' will be recorded in place of an exception message if there is no
 * such message.
 * <br><br>
 * The FlatFileLog also manages file size, shifting oversized files to 
 * a backlog list and keeping the current file below a given size limit.
 * <br><br>
 * There is currently no way of preventing multiple FlatFileLog objects
 * from attempting to write to the same file; any attempt to prevent this
 * within the Log package could be circumvented in some way, so no attempt 
 * is made.
 * <br><br>
 * Author: Reason <br>
 * Last Modified: 12/06/00
 * <br><br>
 * ------------------------------------------------
 * <br><br>                           
 * Copyright (C) 2000 Twilight Minds. All rights reserved.
 * <br><br>
 * This Java package is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * <br><br>
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * <br><br>
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this package; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 * MA 02111-1307 USA
 * <br><br>
 * Alternatively, a copy can be obtained from the Twilight Minds web
 * site: <a href="http://www.twilightminds.com/GNU-LGPL.html">
 * http://www.twilightminds.com/GNU-LGPL.html</a>
 * <br><br>
 * Twilight Minds Design Group
 * <a href="http://www.twilightminds.com/">http://www.twilightminds.com/</a>
 * 
 */
public class FlatFileLog extends Log
{
	
	private String logfilepath;
	private String delimiter;
	private long maxfilesizeinkilobytes;
	
	/**
	 * The standard constructor for a FlatFileLog object. 
	 */
	public FlatFileLog(String inputname, String inputlogfilepath, String inputdelimiter, long inputmaxfilesizeinkilobytes) throws LogException
	{
		super(inputname);
		logfilepath = inputlogfilepath;
		delimiter = inputdelimiter;
		maxfilesizeinkilobytes = inputmaxfilesizeinkilobytes;
		
		// now test the file path to see if it's valid. Throw an
		// exception if not.
		try
		{
			File file = new File(logfilepath);
			
			if(file.isDirectory())
				throw new LogException("File path string '" + logfilepath + "' is a directory, not a file.");
			
			file.createNewFile();
			
			if(!file.canWrite())
				throw new LogException("Cannot write to file '" + logfilepath + "'.");
			
		}
		catch (Exception e)
		{
			throw new LogException("Could not construct FlatFileLog. Exception: " + e.toString());
		}
		
	}
	
	/**
	 * Constructor allowing for specification of the cycle
	 * delay between periods of recording LogEntry objects.
	 */
	public FlatFileLog(String inputname, String inputlogfilepath, String inputdelimiter, long inputmaxfilesizeinkilobytes, long inputcycledelay) throws LogException
	{
		super(inputname, inputcycledelay);
		logfilepath = inputlogfilepath;
		delimiter = inputdelimiter;
		maxfilesizeinkilobytes = inputmaxfilesizeinkilobytes;
		
		// now test the file path to see if it's valid. Throw an
		// exception if not.
		try
		{
			File file = new File(logfilepath);
			
			if(file.isDirectory())
				throw new LogException("File path string '" + logfilepath + "' is a directory, not a file.");
			
			file.createNewFile();
			
			if(!file.canWrite())
				throw new LogException("Cannot write to file '" + logfilepath + "'.");
			
		}
		catch (Exception e)
		{
			throw new LogException("Could not construct FlatFileLog. Exception: " + e.toString());
		}
	}
	
	/**
	 * Return the path for the log file written by this Log.
	 */
	public String getLogFilePath()
	{
		return logfilepath;	
	}
	
	/**
	 * Return the delimiter string for this log.
	 */
	public String getDelimiter()
	{
		return delimiter;	
	}
	
	/**
	 * Return the maximum file size for this log.
	 */
	public long getMaxFileSizeInKiloBytes()
	{
		return maxfilesizeinkilobytes;	
	}

	/**
	 * Output a LogEntry object to the flat file.
	 */
	protected void recordLogEntry(LogEntry logentry) throws Exception
	{
		// attempt to write this logentry to file.
		PrintStream printstream = new PrintStream(new FileOutputStream(logfilepath, true));
		printstream.println(logentry.getDate().toString() + delimiter + logentry.getEntryType().getName() + delimiter + logentry.getEntryMessage() + delimiter + logentry.getExceptionMessage());
		boolean error = printstream.checkError();
		printstream.flush();
		printstream.close();
		
		if(error)
			throw new LogException("Error condition in PrintStream when writing to file '" + logfilepath + "'");
		
		// check on file size. If it's over the mark, then write a new backup file
		// and erase the log file. This isn't going to throw the most informative
		// exceptions -- probably needs another try-catch block wrapped around it.
		File currentfile = new File(logfilepath);
		if(currentfile.length() > maxfilesizeinkilobytes * 1024)
		{
			
			String newfilepath = logfilepath + String.valueOf(System.currentTimeMillis());
		
			File newfile = new File(newfilepath);
			if(!newfile.createNewFile())
				throw new LogException("Attempting to create backup log file '" + newfilepath + "' but file already exists.");
		
			BufferedInputStream bin = new BufferedInputStream(new FileInputStream(logfilepath));
			BufferedOutputStream bout = new BufferedOutputStream(new FileOutputStream(newfilepath));
			int i = 0;
		
			while(true)
			{
				i = bin.read();	
				if(i == -1)
					break;
				bout.write(i);
			}
			
			bin.close();
			bout.flush();
			bout.close();

			currentfile.delete();	
		}
		
	}
	
}
