package TwilightMinds.Log;

import java.util.*;

/**
 * An instance of the Log abstract class provides the functionality to represent a 
 * single log. It stores LogEntry objects that are passed to it and runs as a 
 * thread to record and ease these objects. Information that cannot be recorded 
 * is written to System.err as a last resort. 
 * <br><br>
 * Working Log classes are derived from the Log class by overwriting the private
 * recordLogEntry(LogEntry logentry) method.
 * <br><br>
 * Author: Reason <br>
 * Last Modified: 11/23/00
 * <br><br>
 * ------------------------------------------------
 * <br><br>                           
 * Copyright (C) 2000 Twilight Minds. All rights reserved.
 * <br><br>
 * This Java package is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * <br><br>
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * <br><br>
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this package; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 * MA 02111-1307 USA
 * <br><br>
 * Alternatively, a copy can be obtained from the Twilight Minds web
 * site: <a href="http://www.twilightminds.com/GNU-LGPL.html">
 * http://www.twilightminds.com/GNU-LGPL.html</a>
 * <br><br>
 * Twilight Minds Design Group
 * <a href="http://www.twilightminds.com/">http://www.twilightminds.com/</a>
 * 
 */
public abstract class Log extends Thread
{
	
	private String name;
	private boolean islogging;
	private long cycledelay = 1;
	
	private Vector logentryvector = new Vector();
	
	/**
	 * The standard constructor for a Log object. 
	 */
	public Log(String inputname)
	{
		name = inputname;
		islogging = false;
		
	}
	
	/**
	 * Constructor allowing for specification of the cycle
	 * delay between periods of recording LogEntry objects.
	 */
	public Log(String inputname, long inputcycledelay)
	{
		name = inputname;
		islogging = false;
		cycledelay = inputcycledelay;
		
	}
	
	/**
	 * Return the identifying name of this log.
	 */
	public String getLogName()
	{
		return name;	
	}
	
	/**
	 * Return the state of the log. True indicates that the log is accepting
	 * entries, false that it is not.
	 */
	public boolean isLogging()
	{
		return islogging;	
	}
	
	/**
	 * Set this Log object to accept new LogEntry objects and begin
	 * running as a thread while writing to the log file.
	 */
	public void startLogging()
	{
		try
		{
			islogging = true;
			this.start();
		}
		catch(IllegalThreadStateException  e)
		{
			// not a problem.	
		}
	}
	
	/**
	 * Set this Log object to reject new LogEntry objects. The Log
	 * will write out LogEntry objects that have already been received
	 * and will then stop running as a thread. 
	 */
	public void stopLogging()
	{
		islogging = false;
	}
	
	/**
	 * Return the delay in milliseconds between each cycle
	 * during which the Log writes out pending LogEntry
	 * objects to the log file.
	 */
	public long getCycleDelay()
	{
		return cycledelay;
	}
	
	/**
	 * Set the delay in milliseconds between each cycle
	 * during which the Log writes out pending LogEntry
	 * objects to the log file. 
	 */
	public void setCycleDelay(long inputcycledelay)
	{
		cycledelay = inputcycledelay;		
	}
	
	/**
	 * Add a LogEntry object to the queue waiting to be written.
	 * Returns false if this Log is not accepting new LogEntry
	 * objects.
	 */
	public boolean addLogEntry(LogEntry logentry)
	{
		if(!islogging)
			return false;
		
		logentryvector.addElement(logentry);
		return true;
	}
	
	/**
	 * The run() method for this class runs through the LogEntry
	 * objects waiting to be recorded and deletes these objects as 
	 * this happens. If a LogEntry cannot be recorded for whatever
	 * reason, then the information is written to System.err.
	 */
	public void run()
	{
		islogging = true;
		
		while(islogging)
		{
			
			try
			{
				this.sleep(cycledelay);	
			}
			catch(InterruptedException e)
			{
				
			}
			
			if(!islogging)
				break;
			
			// now examine the waiting LogEntry objects and write them out
			while(logentryvector.size() > 0)
			{
				
				LogEntry logentry = (LogEntry)logentryvector.elementAt(0);
				
				try
				{
					recordLogEntry(logentry);
				}
				catch (Exception e)
				{	
					System.err.println("LOGGING ERROR: in log '" + name + "' generated Exception: " + e.toString() + ". Error occurred while recording LogEntry: TYPE: " + logentry.getEntryType() + " MESSAGE: " + logentry.getEntryMessage() + " EXCEPTION: " + logentry.getExceptionMessage());
				}	
				
				logentryvector.removeElementAt(0);
			
			} // end logentryvector while-loop
		
		} // end islogging while-loop
		
		this.stop();
	}
	
	/**
	 * This abstract method must be overwritten in child classes to provide
	 * an implementation for writing the LogEntry information to a store, 
	 * be that file, database, etc, etc. An Exception must be thrown if
	 * the process of recording the LogEntry fails.
	 */
	protected abstract void recordLogEntry(LogEntry logentry) throws Exception;
	
}
