package TwilightMinds.Log;

import java.io.*;
import java.util.*;

/**
 * The LogManager class keeps track of any aribitrary Log objects and
 * allows easily formatted and synchronized recording of LogEntry objects of
 * various EntryTypes to any of the Logs. All objects of a given EntryType 
 * are sent to a single Log of choice.
 * <br><br>
 * The LogManager class can also enable and disable logging for individual
 * Logs or EntryTypes. 
 * <br><br>
 * Use the static LogManager methods to set up your Logs and EntryTypes
 * when your application initializes, and then you can use the static
 * writeEntry() methods throughout your application.
 * <br><br>
 * Author: Reason <br>
 * Last Modified: 11/23/00
 * <br><br>
 * ------------------------------------------------
 * <br><br>                           
 * Copyright (C) 2000 Twilight Minds. All rights reserved.
 * <br><br>
 * This Java package is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * <br><br>
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * <br><br>
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this package; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 * MA 02111-1307 USA
 * <br><br>
 * Alternatively, a copy can be obtained from the Twilight Minds web
 * site: <a href="http://www.twilightminds.com/GNU-LGPL.html">
 * http://www.twilightminds.com/GNU-LGPL.html</a>
 * <br><br>
 * Twilight Minds Design Group
 * <a href="http://www.twilightminds.com/">http://www.twilightminds.com/</a>
 * 
 */
public abstract class LogManager
{

	private static Hashtable loghashtable = new Hashtable();
	private static Hashtable entrytypehashtable = new Hashtable();
	
	
	private LogManager()
	{
	}
	
	
	/**
	 * Add a new Log to those managed by the LogManager. All Logs
	 * must have unique names; an exception is thrown if a Log by
	 * this name already exists.
	 */
	public static synchronized void addLog(Log log) throws LogException
	{
		if(LogManager.loghashtable.get(log.getLogName()) != null)
			throw new LogException("ERROR: A Log with name '" + log.getLogName() + "' already exists.");

		LogManager.loghashtable.put(log.getLogName(), log);
	}
	
	/**
	 * Add a new Log to those managed by the LogManager. Also add new
	 * EntryTypes that are associated with this Log. All Logs and EntryTypes
	 * must have unique names; an exception is thrown if a Log by
	 * this name already exists or if any of the EntryType names already exist.
	 * Any duplications mean that neither the Log nor the EntryTypes will be
	 * entered.
	 * <br><br>
	 * The new EntryTypes will be set to log.
	 */
	public static synchronized void addLogAndEntryTypes(Log log, String[] entrytypenames) throws LogException
	{
		if(LogManager.loghashtable.get(log.getLogName()) != null)
			throw new LogException("ERROR: A Log with name '" + log.getLogName() + "' already exists.");

		for(int index = 0; index < entrytypenames.length; index++)
		{
			if(LogManager.entrytypehashtable.get(entrytypenames[index]) != null)
				throw new LogException("ERROR: An EntryType with name '" + entrytypenames[index] + "' already exists.");
		}
		
		LogManager.loghashtable.put(log.getLogName(), log);
	
		for(int index = 0; index < entrytypenames.length; index++)
		{
			LogManager.setEntryType(entrytypenames[index], log.getLogName());	
		}
		
	}
	
	/**
	 * Remove this Log from those managed by the LogManager.
	 * This also removes the EntryTypes that wrote to this Log. The names of these 
	 * removed EntryType objects are returned as an array. The array will
	 * be zero-length if no EntryTypes wrote to this Log.
	 */
	public static synchronized String[] removeLog(Log log)
	{
		LogManager.loghashtable.remove(log.getLogName());
		
		// remove the EntryTypes that wrote to this Log.
		Enumeration enumeration = LogManager.entrytypehashtable.keys();
		Vector vector = new Vector();
		
		while(enumeration.hasMoreElements())
		{
			String entrytypename = (String)enumeration.nextElement();
			try
			{
				if(((EntryType)LogManager.entrytypehashtable.get(entrytypename)).getLogName().equals(log.getLogName()))
				{
					vector.addElement(entrytypename);
					LogManager.entrytypehashtable.remove(entrytypename);
				}
			}
			catch (NullPointerException e)
			{
				
			}
		}
		
		String[] entrytypenames;
		
		if(vector.size() == 0)
		{
			entrytypenames = new String[0];
		}
		else
		{
			entrytypenames = new String[vector.size()];
			vector.copyInto(entrytypenames);
		}
		
		return entrytypenames;
		
	}
	
	
	/**
	 * Return an array of the Log names currently in use in the LogManager.
	 */
	public static synchronized String[] getLogNames()
	{
		Enumeration keys = LogManager.loghashtable.keys();
		Vector vector = new Vector();
		
		while(keys.hasMoreElements())
			vector.addElement(keys.nextElement());	
		
		String[] returnarray = new String[vector.size()];
		
		if(vector.size() > 0)
			vector.copyInto(returnarray);
		
		return returnarray;
	}

	/**
	 * Return an array of the Logs currently in use in the LogManager.
	 */
	public static synchronized Log[] getLogs()
	{
		Enumeration elements = LogManager.entrytypehashtable.elements();
		Vector vector = new Vector();
		
		while(elements.hasMoreElements())
			vector.addElement(elements.nextElement());	
		
		Log[] returnarray = new Log[vector.size()];
		
		if(vector.size() > 0)
			vector.copyInto(returnarray);
		
		return returnarray;
	}
	
	/**
	 * Return the Log with this name, or null if no such Log exists.
	 */
	public static synchronized Log getLog(String logname)
	{
		return (Log)LogManager.loghashtable.get(logname);
	}
	
	/**
	 * Add an EntryType that writes to the Log with the given
	 * logname. An exception is thrown if no Log by this name exists.
	 * If an EntryType with this name already exists, it will be replaced
	 * with a new EntryType that writes to this Log.
	 * <br><br>
	 * The new or reset EntryType will have logging set on.
	 */
	public static synchronized void setEntryType(String entrytypename, String logname) throws LogException
	{
		Log log = (Log)LogManager.loghashtable.get(logname);
		if(log == null)
			throw new LogException("ERROR: No Log with name '" + logname + "' exists.");

		LogManager.entrytypehashtable.put(entrytypename, new EntryType(entrytypename, log));	
	}
	
	/**
	 * Add an EntryType that writes to the Log with the given
	 * logname. An exception is thrown if no Log by this name exists.
	 * If an EntryType with this name already exists, it will be replaced
	 * with a new EntryType that writes to this Log.
	 */
	public static synchronized void setEntryType(String entrytypename, String logname, boolean islogging) throws LogException
	{
		Log log = (Log)LogManager.loghashtable.get(logname);
		if(log == null)
			throw new LogException("ERROR: No Log with name '" + logname + "' exists.");

		LogManager.entrytypehashtable.put(entrytypename, new EntryType(entrytypename, log, islogging));	
	}
	
	
	/**
	 * Remove the EntryType with this name.
	 */
	public static synchronized void removeEntryType(String entrytypename)
	{
		LogManager.entrytypehashtable.remove(entrytypename);
	}
	
	/**
	 * Return an array of all EntryType names currently used in the LogManager.
	 * A zero-length array is returned if there are no EntryTypes.
	 */
	public static synchronized String[] getEntryTypeNames()
	{
		Enumeration keys = LogManager.entrytypehashtable.keys();
		Vector vector = new Vector();
		
		while(keys.hasMoreElements())
			vector.addElement(keys.nextElement());	
		
		String[] returnarray = new String[vector.size()];
		
		if(vector.size() > 0)
			vector.copyInto(returnarray);
		
		return returnarray;
		
	}
	
	/**
	 * Return the name of the Log that LogEntries of this EntryType
	 * are sent to. Throw an exception if no such EntryType exists.
	 */
	public static synchronized String sentToLog(String entrytypename) throws LogException
	{
		try
		{
			EntryType entrytype = (EntryType)LogManager.entrytypehashtable.get(entrytypename);
			return entrytype.getLogName();
		}
		catch (NullPointerException e)
		{
			throw new LogException("ERROR: no EntryType exists with name '" + entrytypename + "'.");
		}
	}
	
	/**
	 * Set LogEntries of all EntryTypes to be logged.
	 */
	public static synchronized void startAllEntryTypesLogging()
	{
		Enumeration enumeration = LogManager.entrytypehashtable.elements();
		
		while(enumeration.hasMoreElements())
		{
			EntryType entrytype = (EntryType)enumeration.nextElement();
			entrytype.startLogging();
		}
	}
	
	/**
	 * Stop LogEntries of all EntryTypes from being logged.
	 */
	public static synchronized void stopAllEntryTypesLogging()
	{
		Enumeration enumeration = LogManager.entrytypehashtable.elements();
		
		while(enumeration.hasMoreElements())
		{
			EntryType entrytype = (EntryType)enumeration.nextElement();
			entrytype.stopLogging();
		}
	}
	
	/**
	 * Stop LogEntries with this EntryType from being logged. An exception is
	 * thrown if no EntryType by this name exists.
	 */
	public static synchronized void startEntryTypeLogging(String entrytypename) throws LogException
	{
		try
		{
			EntryType entrytype = (EntryType)entrytypehashtable.get(entrytypename);
			entrytype.startLogging();
		}
		catch (NullPointerException e)
		{
			throw new LogException("ERROR: no EntryType exists with name '" + entrytypename + "'. Cannot start logging for this EntryType.");
		}
	}
	
	/**
	 * Stop logging LogEntries of this EntryType. An exception is
	 * thrown if no EntryType by this name exists.
	 */
	public static synchronized void stopEntryTypeLogging(String entrytypename) throws LogException
	{
		try
		{
			EntryType entrytype = (EntryType)entrytypehashtable.get(entrytypename);
			entrytype.stopLogging();
		}
		catch (NullPointerException e)
		{
			throw new LogException("ERROR: no EntryType exists with name '" + entrytypename + "'. Cannot stop logging for this EntryType.");
		}		
	}
																					
	/**
	 * Start all Logs accepting and recording log entries.
	 */
	public static synchronized void startAllLogs()
	{
		Enumeration enumeration = LogManager.loghashtable.elements();
		
		while(enumeration.hasMoreElements())
		{
			Log log = (Log)enumeration.nextElement();
			log.startLogging();
		}
	}
	
	/**
	 * Stop all Logs from accepting log entries. Existing entries
	 * in the Log queues will still be recorded.
	 */
	public static synchronized void stopAllLogs()
	{
		Enumeration enumeration = LogManager.loghashtable.elements();
		
		while(enumeration.hasMoreElements())
		{
			Log log = (Log)enumeration.nextElement();
			log.stopLogging();
		}
	}
	
	/**
	 * Start this Log accepting LogEntries. An exception is thrown
	 * if no Log by this name exists.
	 */
	public static synchronized void startLog(String logname) throws LogException
	{
		try
		{
			Log log = (Log)LogManager.loghashtable.get(logname);
			log.startLogging();
		}
		catch (NullPointerException e)
		{
			throw new LogException("ERROR: no Log exists with name '" + logname + "'. Cannot start this Log.");
		}
	}
	
	/**
	 * Stop this Log from accepting LogEntries. Existing entries
	 * in the Log queues will still be recorded. An exception is thrown
	 * if no Log by this name exists.
	 */
	public static synchronized void stopLog(String logname) throws LogException
	{
		try
		{
			Log log = (Log)LogManager.loghashtable.get(logname);
			log.stopLogging();
		}
		catch (NullPointerException e)
		{
			throw new LogException("ERROR: no Log exists with name '" + logname + "'. Cannot stop this Log.");
		}
	}
	
	/**
	 * Write an event string and exception as a given EntryType. This method will
	 * write to System.err if a bad EntryType is selected or if creating a new
	 * LogEntry otherwise fails.
	 * <br><br>
	 * This method will return true if this entry was accepted by a Log, or false
	 * if the destination Log is not currently accepting entries or if the
	 * LogEntry is of an EntryType that is not currently being logged.
	 */
	public static synchronized boolean writeEntry(String entrytypename, String entry, Exception exception)
	{
		try
		{
			
			EntryType entrytype = (EntryType)LogManager.entrytypehashtable.get(entrytypename);
			if(entrytype == null)
				throw new LogException("ERROR: No EntryType with name '" + entrytypename + "' exists.");
		
			if(!entrytype.isLogging())
				return false;
			
			LogEntry logentry;
			if(exception == null)	
				logentry = new LogEntry(entrytype, entry);
			else
				logentry = new LogEntry(entrytype, entry, exception);
			
			return entrytype.getLog().addLogEntry(logentry);
			
		}
		catch (Exception e)
		{
			System.err.println((new Date()).toString() + " LOGGING ERROR: while attempting to form new LogEntry with message '" + entry + "'. Exception: " + e.toString()); 
			return false;
		}
	
	}
	
	/**
	 * Write an event string as a given EntryType. This method will
	 * write to System.err if a bad EntryType is selected.
	 * <br><br>
	 * This method will return true if this entry was accepted by a Log, or false
	 * if the destination Log is not currently accepting entries or if the
	 * LogEntry is of an EntryType that is not currently being logged.
	 */
	public static synchronized boolean writeEntry(String entrytypename, String entry)
	{
		return writeEntry(entrytypename, entry, null);
	}
	
}
